/**
 * Imports
 */

'use strict';

exports.__esModule = true;

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { 'default': obj }; }

var _virtex = require('virtex');

var _setAttribute = require('./setAttribute');

var _setAttribute2 = _interopRequireDefault(_setAttribute);

var _removeAttribute = require('./removeAttribute');

var _removeAttribute2 = _interopRequireDefault(_removeAttribute);

var _createElement = require('./createElement');

var _createElement2 = _interopRequireDefault(_createElement);

/**
 * Vars
 */

var _actions$types = _virtex.actions.types;
var CREATE_TEXT_NODE = _actions$types.CREATE_TEXT_NODE;
var CREATE_ELEMENT = _actions$types.CREATE_ELEMENT;
var SET_ATTRIBUTE = _actions$types.SET_ATTRIBUTE;
var REMOVE_ATTRIBUTE = _actions$types.REMOVE_ATTRIBUTE;
var APPEND_CHILD = _actions$types.APPEND_CHILD;
var REPLACE_CHILD = _actions$types.REPLACE_CHILD;
var INSERT_BEFORE = _actions$types.INSERT_BEFORE;
var REMOVE_CHILD = _actions$types.REMOVE_CHILD;

/**
 * Virtex DOM effects driver
 */

function dom(doc) {
  return function (_ref) {
    var dispatch = _ref.dispatch;
    return function (next) {
      return function (action) {
        return handle(doc, dispatch, next, action);
      };
    };
  };
}

function handle(doc, dispatch, next, action) {
  switch (action.type) {
    case CREATE_TEXT_NODE:
      return doc.createTextNode(action.text);
    case CREATE_ELEMENT:
      return _createElement2['default'](doc, dispatch, action.tag, action.attrs, action.children);
    case SET_ATTRIBUTE:
      return _setAttribute2['default'](dispatch, action.node, action.name, action.value);
    case REMOVE_ATTRIBUTE:
      return _removeAttribute2['default'](action.node, action.name, action.value);
    case APPEND_CHILD:
      return action.node.appendChild(action.oldChild);
    case REMOVE_CHILD:
      return action.node.removeChild(action.oldChild);
    case REPLACE_CHILD:
      return action.node.replaceChild(action.newChild, action.oldChild);
    case INSERT_BEFORE:
      return action.node.insertBefore(action.newChild, action.oldChild);
  }

  return next(action);
}

/**
 * Exports
 */

exports['default'] = dom;
module.exports = exports['default'];